%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THREE DEGREE OF FREEDOM, NONDIMENSIONALIZED BOUNCING BALL SIMULATOR

% Written by: Eric Bell        Last modified: 10 September, 2009        Version: 3.4

% ASSUMPTIONS:
%  - Oscillating bar undergoes periodic motions governed by the following position equations:
%            x(t) = A_x*sin(w_x*t+phi_x)    (horizontal location of bar center)
%            z(t) = A_z*sin(w_z*t+phi_z)    (vertical location of bar center)
%            theta(t) = A_th*sin(w_th*t+phi_th)    (angle of bar)
%  - The bar always rotates about its midpoint, independent of the bar's (x,z) coordinates.
%  - The bar is represented by a line, and therefore has no thickness
%  - The ball is represented by a point, and therefore has zero radius
%  - The simulator assumes the surface of the bar is frictionless, however
%        a tangential coefficient of restitution exists.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [P, N] = bBallSim(A_z, A_th, e_n, e_t, phi)

% SYSTEM PARAMETERS
num_impacts = 100;    % Number of impacts simulator will calculate before terminating
plotting = 1;    % 0 or 1 depending on whether trajectory plots are desired 
                    % not recommended for long simulations (>5000 impacts) or sweeps
off = pi-phi;

Amp = [0; A_z; A_th];    % Position amplitude of bar [A_x, A_z, A_th]
Phase = [0; off+phi; off/2];    % Phase angle of bar [phi_x, phi_z, phi_th]
f = [2; 2; 1];    % Frequencies of bar motions [f_x, f_z, f_th]
w = 2*pi.*f;    % [w_x, w_z, w_th]

% INITIAL CONDITIONS / INITIALIZATIONS
initCond = [0, 0, 0, 0, 0.1223, 0];    % [t0, x0, z0, Vx0, Vz0, locked]
P = zeros(num_impacts, 6);    % Initialize P
launched = 0;    % If locking occurs, indicates whether ball is relaunched
out_of_bounds = 0;    % Terminates simulation if ball goes outside predefined boundaries
terminal = 0;    % Terminates simulation if ball's velocity decays to zero
index = 1;

% DEFINED CONSTANTS
g = 1;    % Gravitational Acceleration

preImpact = initCond;
for i = [1 : num_impacts]
    if (index ~= 1)
        preImpact = P(index - 1, :);
    end
    % Calculate time at which impact occurs
    [t2, out_of_bounds, del] = calculateIntersection(preImpact, Amp, Phase, w, g, launched);
    launched = 0;
    % Check to see if locking occurs
    islocked = checkLocking(preImpact(1), t2);
    % Calculate conditions at imact, save data in the matrix 'P'
    P(index, :) = impact(preImpact, t2, Amp, Phase, w, g, e_n, e_t, islocked, del);
    % If locking occurs, calculate the launch conditions
    if (islocked)
        index = index + 1;
        [P(index, :), terminal, launched] = calculateLaunch(P(index-1, :), Amp, Phase, w, g);
    end 
    if (terminal || out_of_bounds)
        break
        switch (1)    
            case terminal    % If locking occurs and bar's vertical acceleration is less than g,
                                 % ball will never re-launch. Simulation terminates.
                disp('Ball velocity decayed to zero. Bar amplitude less than g.')
            case out_of_bounds    % If ball goes beyond predefined boundaries, simulation terminates.
                disp('Ball out of bounds. Simulation terminated.')
        end
    end
    index = index + 1; 
end

% Return matrix of trajectories if indicated
if (nargout > 1)
    N = plotTraj(P, initCond, Amp, Phase, w, g, terminal, out_of_bounds, index - 1, plotting);
end
