%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Uses binary search method to find time at which ball intersects the bar,
%     based off of equations of motion.
% Returns the time of impact (t2), an indicator as to whether the ball has gone
%     out of bounds (out_of_bounds), and the actual difference in height between the ball
%     and the bar for the given time (del).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t2, out_of_bounds, del] = calculateIntersection(preImpact, Amp, phi, w, g, launched)

    % SOLVER PARAMETERS
    time = (15e-8);   % time since t1
    step = (3*preImpact(5)/10);
    if (preImpact(5)/10 < (0.0001))
        step = (.0015);
    end
    tol = (23e-10);
    out_of_bounds = 0;
    
    % If ball is in launch condition, initial time should be greater, to avoid
        % Accidental double impact
    if (launched)
        time = 0.2;    % One-tenth of a period
    end
    [del, R] = calcDel(preImpact, Amp, phi, w, time, g);
    % Until change in height is less than a certain threshold...
    while (abs(del) > tol)
        % Converges on intersection point
        while sign(step)*del > 0
            time = time + step;
            [del, R] = calcDel(preImpact, Amp, phi, w, time, g);
        end
        step = -step/2;
    end
    t2 = preImpact(1) + time;
    if (R > 115)
        out_of_bounds = 1;
    end
    if (time < 0)
        error('calculateIntersection returned time < 0')
    end 
end

% Returns the difference in height between the bar and the ball
function [del, R] = calcDel(preImpact, Amp, Phase, w, time, g)
    BAR = Amp.*sin(w*(time + preImpact(1)) + Phase);
    BALL = [ preImpact(4)*time + preImpact(2); -0.5*g*time^2 + preImpact(5)*time + preImpact(3) ];
    R = [ BALL(1) - BAR(1) ] / cos(BAR(3));
    del = BALL(2) - BAR(2) - R*sin(BAR(3));       
end
